@@ -30,6 +30,7 @@ class _TMotorManState_Servo(Enum):
     POSITION = 4
     SET_ORIGIN=5
     POSITION_VELOCITY=6
+    IDLE = 7
 
 # the user-facing class that manages the motor.
 class TMotorManager_servo_can():
@@ -68,7 +69,7 @@ class TMotorManager_servo_can():
         self._motor_state = servo_motor_state(0.0,0.0,0.0,0.0,0.0,0.0)
         self._motor_state_async = servo_motor_state(0.0,0.0,0.0,0.0,0.0,0.0)
         self._command = servo_command(0.0,0.0,0.0,0.0)
-        self._control_state = _TMotorManState_Servo.DUTY_CYCLE
+        self._control_state = _TMotorManState_Servo.IDLE
         self._times_past_position_limit = 0
         self._times_past_current_limit = 0
         self._times_past_velocity_limit = 0
@@ -80,6 +81,7 @@ class TMotorManager_servo_can():
         self._old_vel = 0.0
         self._old_current_zone = 0
         self.radps_per_ERPM = 5.82E-04
+        self.rad_per_Eang = np.pi/Servo_Params[self.type]['NUM_POLE_PAIRS'] # 2*(np.pi/180)/(Servo_Params[self.type]['NUM_POLE_PAIRS'])
 
         self._entered = False
         self._start_time = time.time()
@@ -155,15 +157,18 @@ class TMotorManager_servo_can():
             raise RuntimeError('Driver board error for device: ' + self.device_info_string() + ": " + Servo_Params['ERROR_CODES'][servo_state.error])
 
         now = time.time()
+        dt = self._last_update_time - now
         self._last_update_time = now
-        # dt = self._last_update_time - now
-        # acceleration = (servo_state.velocity - self._motor_state_async.velocity)/dt
-
+        # print(f"async: {dt}")
+        self._motor_state_async.acceleration = (servo_state.velocity - self._motor_state_async.velocity)/dt
         # The "Current" supplied by the controller is actually current*Kt, which approximates torque.
         self._motor_state_async.set_state_obj(servo_state)
         
         self._updated = True
 
+        # # send current motor command
+        # self._send_command()
+
     
     # this method is called by the user to synchronize the current state used by the controller
     # with the most recent message recieved
@@ -182,6 +187,7 @@ class TMotorManager_servo_can():
         # check that the motor data is recent
         # print(self._command_sent)
         now = time.time()
+        # print(f"update: {now - self._last_command_time}")
         if (now - self._last_command_time) < 0.25 and ( (now - self._last_update_time) > 0.1):
             # print("State update requested but no data recieved from motor. Delay longer after zeroing, decrease frequency, or check connection.")
             warnings.warn("State update requested but no data from motor. Delay longer after zeroing, decrease frequency, or check connection. " + self.device_info_string(), RuntimeWarning)
@@ -219,6 +225,9 @@ class TMotorManager_servo_can():
             self._canman.comm_can_set_rpm(self.ID, self._command.velocity)
         elif self._control_state == _TMotorManState_Servo.POSITION:
             self._canman.comm_can_set_pos(self.ID, self._command.position)
+        elif self._control_state == _TMotorManState_Servo.IDLE:
+            self._canman.comm_can_set_duty(self.ID, 0.0)
+
         #TODO:Add other modes
         else:
             raise RuntimeError("UNDEFINED STATE for device " + self.device_info_string())
@@ -237,10 +246,10 @@ class TMotorManager_servo_can():
 
     # zeros the position, like a scale you have to wait about a second before you can
     # use the motor again. This responsibility is on the user!!
-    def zero_position(self):
+    def set_zero_position(self):
         """Zeros the position--like a scale you have to wait about a second before you can
         use the motor again. This responsibility is on the user!!"""
-        self._canman.zero(self.ID)
+        self._canman.comm_can_set_origin(self.ID,1)
         self._last_command_time = time.time()
 
     # getters for motor state
@@ -281,7 +290,7 @@ class TMotorManager_servo_can():
         Returns:
         The most recently updated output angle in radians
         """
-        return self._motor_state.position*np.pi/180
+        return self._motor_state.position*self.rad_per_Eang
 
     def get_output_velocity_radians_per_second(self):
         """
@@ -322,6 +331,9 @@ class TMotorManager_servo_can():
     def enter_position_velocity_control(self):
         self._control_state = _TMotorManState_Servo.POSITION_VELOCITY
 
+    def enter_idle_mode(self):
+        self._control_state = _TMotorManState_Servo.IDLE
+
     # used for either impedance or MIT mode to set output angle
     def set_output_angle_radians(self, pos):
         """
@@ -335,9 +347,9 @@ class TMotorManager_servo_can():
         if np.abs(pos) >= Servo_Params[self.type]["P_max"]:
             raise RuntimeError("Cannot control using impedance mode for angles with magnitude greater than " + str(Servo_Params[self.type]["P_max"]) + "rad!")
         if self._control_state == _TMotorManState_Servo.POSITION:
-            self._command.position = pos*np.pi/180
+            self._command.position = pos/self.rad_per_Eang
 
-    def set_duty_cycle(self, duty):
+    def set_duty_cycle_percent(self, duty):
         if self._control_state not in [_TMotorManState_Servo.DUTY_CYCLE]:
             raise RuntimeError("Attempted to send duty cycle command without gains for device " + self.device_info_string()) 
         else:
@@ -420,7 +432,7 @@ class TMotorManager_servo_can():
         Returns:
             The most recently updated motor-side angle in rad.
         """
-        return self._motor_state.position*np.pi/180*Servo_Params[self.type]["GEAR_RATIO"]
+        return self._motor_state.position*self.rad_per_Eang*Servo_Params[self.type]["GEAR_RATIO"]
 
     def get_motor_velocity_radians_per_second(self):
         """
@@ -452,7 +464,7 @@ class TMotorManager_servo_can():
     # Pretty stuff
     def __str__(self):
         """Prints the motor's device info and current"""
-        return self.device_info_string() + " | Position: " + '{: 1f}'.format(round(self.θ,3)) + " rad | Velocity: " + '{: 1f}'.format(round(self.θd,3)) + " rad/s | current: " + '{: 1f}'.format(round(self.i,3)) + " A | temp: " + '{: 1f}'.format(round(self.T,0)) + " C"
+        return self.device_info_string() + " | Position: " + '{: 1f}'.format(round(self.position,3)) + " rad | Velocity: " + '{: 1f}'.format(round(self.velocity,3)) + " rad/s | current: " + '{: 1f}'.format(round(self.current_qaxis,3)) + " A | temp: " + '{: 1f}'.format(round(self.temperature,0)) + " C"
 
     def device_info_string(self):
         """Prints the motor's ID and device type."""
@@ -482,51 +494,41 @@ class TMotorManager_servo_can():
         # self._canman.notifier.remove_listener(Listener)
         return success
 
-   
-
-
-
-    ## Greek letter math symbol property interface. This is the good
-    #  interface, for those who like code that resembles math. It works best
-    #  to use the UnicodeMath plugin for sublime-text, "Fast Unicode Math
-    #  Characters" in VS Code, or the like to allow easy typing of ϕ, θ, and
-    #  τ.
-
     # controller variables
-    T = property(get_temperature_celsius, doc="temperature_degrees_C")
+    temperature = property(get_temperature_celsius, doc="temperature_degrees_C")
     """Temperature in Degrees Celsius"""
 
-    e = property(get_motor_error_code, doc="temperature_degrees_C")
+    error = property(get_motor_error_code, doc="temperature_degrees_C")
     """Motor error code. 0 means no error."""
 
     # electrical variables
-    i = property(get_current_qaxis_amps, set_motor_current_qaxis_amps, doc="current_qaxis_amps_current_only")
+    current_qaxis = property(get_current_qaxis_amps, set_motor_current_qaxis_amps, doc="current_qaxis_amps_current_only")
     """Q-axis current in amps"""
 
     # output-side variables
-    θ = property(get_output_angle_radians, set_output_angle_radians, doc="output_angle_radians_impedance_only")
+    position = property(get_output_angle_radians, set_output_angle_radians, doc="output_angle_radians_impedance_only")
     """Output angle in rad"""
 
-    θd = property (get_output_velocity_radians_per_second, set_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
+    velocity = property (get_output_velocity_radians_per_second, set_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
     """Output velocity in rad/s"""
 
-    θdd = property(get_output_acceleration_radians_per_second_squared, doc="output_acceleration_radians_per_second_squared")
+    acceleration = property(get_output_acceleration_radians_per_second_squared, doc="output_acceleration_radians_per_second_squared")
     """Output acceleration in rad/s/s"""
 
-    τ = property(get_output_torque_newton_meters, set_output_torque_newton_meters, doc="output_torque_newton_meters")
+    torque = property(get_output_torque_newton_meters, set_output_torque_newton_meters, doc="output_torque_newton_meters")
     """Output torque in Nm"""
 
     # motor-side variables
-    ϕ = property(get_motor_angle_radians, set_motor_angle_radians, doc="motor_angle_radians_impedance_only")
+    angle_motorside = property(get_motor_angle_radians, set_motor_angle_radians, doc="motor_angle_radians_impedance_only")
     """Motor-side angle in rad"""
     
-    ϕd = property (get_motor_velocity_radians_per_second, set_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
+    velocity_motorside = property (get_motor_velocity_radians_per_second, set_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
     """Motor-side velocity in rad/s"""
 
-    ϕdd = property(get_motor_acceleration_radians_per_second_squared, doc="motor_acceleration_radians_per_second_squared")
+    acceleration_motorside = property(get_motor_acceleration_radians_per_second_squared, doc="motor_acceleration_radians_per_second_squared")
     """Motor-side acceleration in rad/s/s"""
 
-    τm = property(get_motor_torque_newton_meters, set_motor_torque_newton_meters, doc="motor_torque_newton_meters")
+    torque_motorside = property(get_motor_torque_newton_meters, set_motor_torque_newton_meters, doc="motor_torque_newton_meters")
     """Motor-side torque in Nm"""
 
 
