@@ -2,7 +2,7 @@ import serial
 from serial import threaded
 import time
 import numpy as np
-from NeuroLocoMiddleware.SoftRealtimeLoop import SoftRealtimeLoop
+from NeuroLocoMiddleware.StatProfiler import SSProfile
 from enum import Enum
 import traceback
 import csv
@@ -481,6 +481,7 @@ class motor_listener(serial.threaded.Protocol):
         Args:
             data: array of received data to parse
         """
+        # print(f"\n {len(data)}")
         for d in data:
             if self.state == 0:
                 if d == 0x02:
@@ -514,7 +515,6 @@ class motor_listener(serial.threaded.Protocol):
         """
         if self.motor is not None:
             # print(packet)
-            header = packet[0]
             DL = packet[1]
             data = packet[2:2+DL]
             crc = buffer_get_int16(packet[2+DL:DL+4], 0)
@@ -568,7 +568,15 @@ class TMotorManager_servo_serial():
         """
         if not self._entered:
             # begin serial connection
-            self._ser = serial.Serial(self.port, self.baud)
+            self._ser = serial.Serial(
+                self.port, 
+                self.baud, 
+                parity=serial.PARITY_NONE,
+                stopbits=serial.STOPBITS_ONE,
+                bytesize=serial.EIGHTBITS,
+                timeout=0.0001, 
+                inter_byte_timeout=0.001
+                )
 
             # start another thread to handle recieved data
             self._reader_thread = serial.threaded.ReaderThread(self._ser, motor_listener)
@@ -597,8 +605,6 @@ class TMotorManager_servo_serial():
             else:
                 print(f"device: {self.device_info_string()} successfully connected.")
 
-            self.f = open('timing_test_log.csv','w').__enter__()
-            self.w = csv.writer(self.f)
             # return the safely entered motor manager object
             self._entered = True
             return self
@@ -648,7 +654,7 @@ class TMotorManager_servo_serial():
         """
         self._updated_async = True
         packet_ID = data[0]
-        if (packet_ID == COMM_PACKET_ID.COMM_GET_VALUES) or (packet_ID == COMM_PACKET_ID.COMM_GET_VALUES_SETUP):
+        if (packet_ID in [COMM_PACKET_ID.COMM_GET_VALUES, COMM_PACKET_ID.COMM_GET_VALUES_SETUP]):
             self.parse_motor_parameters_async(data)
             # calculate acceleration
             now = time.time()
@@ -657,9 +663,73 @@ class TMotorManager_servo_serial():
             self._last_update_time = now
         elif (packet_ID == COMM_PACKET_ID.COMM_ROTOR_POSITION):
             self.parse_position_feedback_async(data)
+        else:
+            print("weird packet")
+        # elif (packet_ID == COMM_PACKET_ID.COMM_SET_POS):
+        #     self.parse_set_position_feedback_async(data)
 
         if self._motor_state.error != 0:
             raise RuntimeError(ERROR_CODES[self._motor_state.error])
+
+        
+
+    # comm data parsing 
+    def parse_position_feedback_async(self, data):
+        """
+        Update this motor's asynch position based on recieved data
+
+        Args:
+            data: The data array to parse the position from.
+        """
+        self._motor_state_async.position = -float(buffer_get_int32(data, 1))/1000.0
+
+    def parse_set_position_feedback_async(self, data):
+        """
+        Update this motor's asynch position based on recieved data.
+
+        Args:
+            data: The data array to parse the position from.
+        """
+        # not sure why the constant varies here, maybe the streaming data is more accurate?
+        self._motor_state_async.position = -float(buffer_get_int32(data, 1))/1000000.0
+
+    def parse_motor_parameters_async(self, data):
+        """
+        Update this motor's asynch state (except position) based on received data
+ 
+        Args:
+            data: The data array to parse the parameters from
+        """
+        i = 1
+        self._motor_state_async.mos_temperature = float(buffer_get_int16(data,i))/10.0
+        i+=2
+        self._motor_state_async.motor_temperature = float(buffer_get_int16(data,i))/10.0
+        i+=2
+        self._motor_state_async.output_current = float(buffer_get_int32(data,i))/100.0
+        i+=4
+        self._motor_state_async.input_current = float(buffer_get_int32(data,i))/100.0
+        i+=4
+        self._motor_state_async.id_current = float(buffer_get_int32(data,i))/100.0
+        i+=4
+        self._motor_state_async.iq_current = float(buffer_get_int32(data,i))/100.0
+        i+=4
+        self._motor_state_async.duty = float(buffer_get_int16(data,i))/1000.0
+        i+=2
+        self._motor_state_async.speed = float(buffer_get_int32(data,i))
+        i+=4
+        self._motor_state_async.input_voltage = float(buffer_get_int16(data,i))/10.0
+        i+=2 + 24
+        # TODO investigate what's in the 24 reserved bytes? Maybe it's interesting to record?
+        self._motor_state_async.error = np.uint(data[i])
+        i+=1
+        self._motor_state_async.position_set = float(buffer_get_int32(data,i))/1000000.0
+        i+=4
+        self._motor_state_async.controlID = np.uint(data[i])
+        i+=1 + 6
+        self._motor_state_async.Vd = float(buffer_get_int32(data,i))/1000.0
+        i+=4
+        self._motor_state_async.Vq = float(buffer_get_int32(data,i))/1000.0
+        i+=4
     
     def send_command(self):
         """
@@ -670,6 +740,7 @@ class TMotorManager_servo_serial():
             # object while the reader thread is using it!!
             # TODO when pyserial adds full asyncio support, consider switching to that
             self._reader_thread.write(self._command)
+            
 
     def _send_specific_command(self, command):
         """
@@ -701,13 +772,12 @@ class TMotorManager_servo_serial():
 
         # send the command to get parameters (message will be read in other thread)
         self._send_specific_command(self.get_motor_parameters())
+        
 
         # synchronize user-facing state with most recent async state
         # TODO implement some filtering on the async state, if it gets multiple updates
         # between user requested updates
         self._motor_state = self._motor_state_async
-
-        # self.w.writerow([time.time()-self._start_time])
         
     # comm protocol commands
     def power_on(self):
@@ -758,7 +828,7 @@ class TMotorManager_servo_serial():
         """
         self._control_state = SERVO_SERIAL_CONTROL_STATE.POSITION_VELOCITY
 
-    def enter_current_loop_control(self):
+    def enter_current_control(self):
         """
         Set the control state to CURENT_LOOP
         In this mode, you can send an Iq current to motor, the motor output torque = Iq *KT, so it can
@@ -773,7 +843,7 @@ class TMotorManager_servo_serial():
         """
         self._control_state = SERVO_SERIAL_CONTROL_STATE.DUTY_CYCLE
 
-    def set_duty_cycle(self, duty, set_command=True):
+    def comm_set_duty_cycle(self, duty, set_command=True):
         """
         send a certain duty cycle voltage to motor
 
@@ -791,7 +861,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray(create_packet(data))
         return self._command
 
-    def set_speed_ERPM(self, speed, set_command=True):
+    def comm_set_speed_ERPM(self, speed, set_command=True):
         """
         send a certain motion speed to motor
 
@@ -809,7 +879,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray(create_packet(data))
         return self._command
 
-    def set_current_loop(self, current, set_command=True):
+    def comm_set_current_loop(self, current, set_command=True):
         """
         send am Iq current to motor, the motor output torque = Iq *KT, so it can
         be used as a torque loop
@@ -828,7 +898,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray(create_packet(data))
         return self._command
 
-    def set_position(self, pos, set_command=True):
+    def comm_set_position(self, pos, set_command=True):
         """
         send a certain position to motor, the motor will run to the specified
         position, (default speed 12000erpm acceleration 40000erpm)
@@ -847,7 +917,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray(create_packet(data))
         return self._command
 
-    def set_position_velocity(self, pos, vel, acc, set_command=True):
+    def comm_set_position_velocity(self, pos, vel, acc, set_command=True):
         """
         send a certain position, speed and acceleration to motor.
         The motor will run at a given acceleration and maximum speed to a specified
@@ -872,7 +942,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray(create_packet(data))
         return self._command
 
-    def set_multi_turn(self, set_command=True):
+    def comm_set_multi_turn(self, set_command=True):
         """
         Tell the motor to operate in multi-turn mode, rather than being limited to 
         Just 360 degrees of position feedback.
@@ -903,7 +973,7 @@ class TMotorManager_servo_serial():
             self._command = bytearray([0x02, 0x02, 0x5F, 0x01, 0x0E, 0xA0, 0x03])
         return cmd
     
-    def set_motor_parameter_return_format_all(self, set_command=True):
+    def comm_set_motor_parameter_return_format_all(self, set_command=True):
         """
         Tell the motor to send back all possible fields when an update is requested.
 
@@ -920,7 +990,7 @@ class TMotorManager_servo_serial():
             self._command = cmd
         return cmd
 
-    def begin_position_feedback(self, set_command=True):
+    def comm_begin_position_feedback(self, set_command=True):
         """
         Tell the motor to send back its current position every 10ms
 
@@ -933,7 +1003,7 @@ class TMotorManager_servo_serial():
         if set_command:
             self._command = bytearray([0x02, 0x02, 0x0B, 0x04, 0x9C, 0x7E, 0x03])
 
-    def get_motor_parameters(self, set_command=True):
+    def comm_get_motor_parameters(self, set_command=True):
         """
         Request the current motor parameters
 
@@ -950,54 +1020,6 @@ class TMotorManager_servo_serial():
             self._command = cmd
         return cmd
         
-    # comm data parsing 
-    def parse_position_feedback_async(self, data):
-        """
-        Update this motor's asynch position based on recieved data
-
-        Args:
-            data: The data array to parse the position from.
-        """
-        self._motor_state_async.position = -float(buffer_get_int32(data, 1))/1000.0
-
-    def parse_motor_parameters_async(self, data):
-        """
-        Update this motor's asynch state (except position) based on received data
- 
-        Args:
-            data: The data array to parse the parameters from
-        """
-        i = 1
-        self._motor_state_async.mos_temperature = float(buffer_get_int16(data,i))/10.0
-        i+=2
-        self._motor_state_async.motor_temperature = float(buffer_get_int16(data,i))/10.0
-        i+=2
-        self._motor_state_async.output_current = float(buffer_get_int32(data,i))/100.0
-        i+=4
-        self._motor_state_async.input_current = float(buffer_get_int32(data,i))/100.0
-        i+=4
-        self._motor_state_async.id_current = float(buffer_get_int32(data,i))/100.0
-        i+=4
-        self._motor_state_async.iq_current = float(buffer_get_int32(data,i))/100.0
-        i+=4
-        self._motor_state_async.duty = float(buffer_get_int16(data,i))/1000.0
-        i+=2
-        self._motor_state_async.speed = float(buffer_get_int32(data,i))
-        i+=4
-        self._motor_state_async.input_voltage = float(buffer_get_int16(data,i))/10.0
-        i+=2 + 24
-        # TODO investigate what's in the 24 reserved bytes? Maybe it's interesting to record?
-        self._motor_state_async.error = np.uint(data[i])
-        i+=1
-        self._motor_state_async.position_set = float(buffer_get_int32(data,i))/1000000.0
-        i+=4
-        self._motor_state_async.controlID = np.uint(data[i])
-        i+=1 + 6
-        self._motor_state_async.Vd = float(buffer_get_int32(data,i))/1000.0
-        i+=4
-        self._motor_state_async.Vq = float(buffer_get_int32(data,i))/1000.0
-        i+=4
-
     # getters for motor state
     def get_temperature_celsius(self):
         """
@@ -1036,6 +1058,9 @@ class TMotorManager_servo_serial():
         """
         return self._motor_state.error
 
+    def get_motor_error_string(self):
+        return ERROR_CODES[self._motor_state.error]
+
     def get_current_qaxis_amps(self):
         """
         Returns:
@@ -1122,7 +1147,7 @@ class TMotorManager_servo_serial():
         if self._control_state not in [SERVO_SERIAL_CONTROL_STATE.VELOCITY]:
             raise RuntimeError("Attempted to send speed command without entering speed control " + self.device_info_string()) 
 
-        self.set_speed_ERPM(vel/self.radps_per_ERPM)
+        self.comm_set_speed_ERPM(vel/self.radps_per_ERPM)
 
     def set_duty_cycle_percent(self, duty):
         """
@@ -1139,7 +1164,7 @@ class TMotorManager_servo_serial():
         if self._control_state not in [SERVO_SERIAL_CONTROL_STATE.DUTY_CYCLE]:
             raise RuntimeError("Attempted to duty cycle command without entering duty cycle control " + self.device_info_string()) 
 
-        self.set_duty_cycle(duty)
+        self.comm_set_duty_cycle(duty)
 
     def set_motor_current_qaxis_amps(self, curr):
         """
@@ -1156,7 +1181,7 @@ class TMotorManager_servo_serial():
         if self._control_state not in [SERVO_SERIAL_CONTROL_STATE.CURRENT_LOOP]:
             raise RuntimeError("Attempted to send current command without entering current control " + self.device_info_string()) 
 
-        self.set_current_loop(curr)
+        self.comm_set_current_loop(curr)
 
     def set_output_angle_radians(self, pos, vel=1000, acc=500):
         """
@@ -1173,11 +1198,11 @@ class TMotorManager_servo_serial():
         if np.abs(pos) >= self.motor_params["P_max"]:
             raise RuntimeError("Cannot control using position mode for angles with magnitude greater than " + str(self.motor_params["P_max"]) + "rad!")
 
-        pos = (pos * 180 / np.pi)
-        if self._control_state == SERVO_SERIAL_CONTROL_STATE.POSITION:
-            self.set_position_velocity(pos, vel, acc)
-        elif self._control_state == SERVO_SERIAL_CONTROL_STATE.POSITION_VELOCITY:
-            self.set_position(pos)
+        pos = (pos / self.rad_per_Eang)
+        if self._control_state == SERVO_SERIAL_CONTROL_STATE.POSITION_VELOCITY:
+            self.comm_set_position_velocity(pos, vel, acc)
+        elif self._control_state == SERVO_SERIAL_CONTROL_STATE.POSITION:
+            self.comm_set_position(pos)
         else:
             raise RuntimeError("Attempted to send position command without entering position control " + self.device_info_string()) 
 
@@ -1259,63 +1284,83 @@ class TMotorManager_servo_serial():
     # Pretty stuff
     def __str__(self):
         """Prints the motor's device info and current state"""
-        return self.device_info_string() + " | Position: " + '{: 1f}'.format(round(self.get_output_angle_radians(),3)) + " rad | Velocity: " + '{: 1f}'.format(round(self.get_output_velocity_radians_per_second(),3)) + " rad/s | current: " + '{: 1f}'.format(round(self._motor_state.iq_current,3)) + " A | temp: " + '{: 1f}'.format(round(self._motor_state.mos_temperature,0)) + " C"
+        return self.device_info_string() + " | Position: " + '{: 1f}'.format(round(self.get_output_angle_radians(),3)) + " rad | Velocity: " + '{: 1f}'.format(round(self.get_output_velocity_radians_per_second(),3)) + " rad/s | current: " + '{: 1f}'.format(round(self._motor_state.iq_current,3)) + " A | temp: " + '{: 1f}'.format(round(self._motor_state.mos_temperature,0)) + " C"  + f' | Error: {self.get_motor_error_string()}'
 
     def device_info_string(self):
         """Prints the motor's serial port and device type."""
         return f"{self.motor_params['Type']} Port: {self.port}"
 
     # controller variables
-    T = property(get_temperature_celsius, doc="temperature_degrees_C")
+    temperature = property(get_temperature_celsius, doc="temperature_degrees_C")
     """Temperature in Degrees Celsius"""
 
     # TODO write actual codes in description here as well
     error = property(get_motor_error_code, doc="Error")
-    """Motor error code. 0 means no error."""
+    """Error Codes:
+            0  : 'FAULT_CODE_NONE'
+            1  : 'FAULT_CODE_OVER_VOLTAGE'
+            2  : 'FAULT_CODE_UNDER_VOLTAGE'
+            3  : 'FAULT_CODE_DRIVE'
+            4  : 'FAULT_CODE_ABS_OVER_CURRENT'
+            5  : 'FAULT_CODE_OVER_TEMP_FET'
+            6  : 'FAULT_CODE_OVER_TEMP_MOTOR'
+            7  : 'FAULT_CODE_GATE_DRIVER_OVER_VOLTAGE'
+            8  : 'FAULT_CODE_GATE_DRIVER_UNDER_VOLTAGE'
+            9  : 'FAULT_CODE_MCU_UNDER_VOLTAGE'
+            10 : 'FAULT_CODE_BOOTING_FROM_WATCHDOG_RESET'
+            11 : 'FAULT_CODE_ENCODER_SPI'
+            12 : 'FAULT_CODE_ENCODER_SINCOS_BELOW_MIN_AMPLITUDE'
+            13 : 'FAULT_CODE_ENCODER_SINCOS_ABOVE_MAX_AMPLITUDE'
+            14 : 'FAULT_CODE_FLASH_CORRUPTION'
+            15 : 'FAULT_CODE_HIGH_OFFSET_CURRENT_SENSOR_1'
+            16 : 'FAULT_CODE_HIGH_OFFSET_CURRENT_SENSOR_2'
+            17 : 'FAULT_CODE_HIGH_OFFSET_CURRENT_SENSOR_3'
+            18 : 'FAULT_CODE_UNBALANCED_CURRENTS'
+    """
 
     # electrical variables
-    iq = property(get_current_qaxis_amps, set_motor_current_qaxis_amps, doc="current_qaxis_amps")
+    current_qaxis = property(get_current_qaxis_amps, set_motor_current_qaxis_amps, doc="current_qaxis_amps")
     """Q-axis current in amps"""
 
-    id = property(get_current_daxis_amps, doc="current_daxis_amps")
+    current_daxis = property(get_current_daxis_amps, doc="current_daxis_amps")
     """D-axis current in amps"""
 
-    ibus = property(get_current_bus_amps, doc="current_bus_amps")
+    current_bus = property(get_current_bus_amps, doc="current_bus_amps")
     """Bus input current in amps"""
 
-    vq = property(get_voltage_qaxis_volts, doc="voltage_qaxis_volts")
+    voltage_qaxis = property(get_voltage_qaxis_volts, doc="voltage_qaxis_volts")
     """Q-axis voltage in volts"""
 
-    vd = property(get_voltage_daxis_volts, doc="voltage_daxis_volts")
+    voltage_daxis = property(get_voltage_daxis_volts, doc="voltage_daxis_volts")
     """D-axis voltage in volts"""
 
-    vbus = property(get_voltage_bus_volts, doc="voltage_bus_volts")
+    voltage_bus = property(get_voltage_bus_volts, doc="voltage_bus_volts")
     """Bus input voltage in volts"""
 
     # output-side variables
-    θ = property(get_output_angle_radians, set_output_angle_radians, doc="output_angle_radians")
+    position = property(get_output_angle_radians, set_output_angle_radians, doc="output_angle_radians")
     """Output angle in rad"""
 
-    θd = property (get_output_velocity_radians_per_second, set_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
+    velocity = property (get_output_velocity_radians_per_second, set_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
     """Output velocity in rad/s"""
 
-    θdd = property(get_output_acceleration_radians_per_second_squared, doc="output_acceleration_radians_per_second_squared")
+    acceleration = property(get_output_acceleration_radians_per_second_squared, doc="output_acceleration_radians_per_second_squared")
     """Output acceleration in rad/s/s"""
 
-    τ = property(get_output_torque_newton_meters, set_output_torque_newton_meters, doc="output_torque_newton_meters")
+    torque = property(get_output_torque_newton_meters, set_output_torque_newton_meters, doc="output_torque_newton_meters")
     """Output torque in Nm"""
 
     # motor-side variables
-    ϕ = property(get_motor_angle_radians, set_motor_angle_radians, doc="motor_angle_radians")
+    angle_motorside = property(get_motor_angle_radians, set_motor_angle_radians, doc="motor_angle_radians")
     """Motor-side angle in rad"""
     
-    ϕd = property (get_motor_velocity_radians_per_second, set_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
+    velocity_motorside = property (get_motor_velocity_radians_per_second, set_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
     """Motor-side velocity in rad/s"""
 
-    ϕdd = property(get_motor_acceleration_radians_per_second_squared, doc="motor_acceleration_radians_per_second_squared")
+    acceleration_motorside = property(get_motor_acceleration_radians_per_second_squared, doc="motor_acceleration_radians_per_second_squared")
     """Motor-side acceleration in rad/s/s"""
 
-    τm = property(get_motor_torque_newton_meters, set_motor_torque_newton_meters, doc="motor_torque_newton_meters")
+    torque_motorside = property(get_motor_torque_newton_meters, set_motor_torque_newton_meters, doc="motor_torque_newton_meters")
     """Motor-side torque in Nm"""
 
 
